//head:-------------------------------------------------------------------------
//------------------------------------------------------------------------------
// $Id: virtual_ptr.h,v 1.2 2011/05/13 13:09:32 twagner Exp $
//------------------------------------------------------------------------------
// NO WARRANTIES: TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER THE
// INTELLECTUAL PROPERTY OWNERS, COPYRIGHT HOLDERS AND CONTRIBUTORS, NOR ANY
// PERSON, EITHER EXPRESSLY OR IMPLICITLY, WARRANTS ANY ASPECT OF THIS
// SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR PROGRAM RELATED THERETO,
// INCLUDING ANY OUTPUT OR RESULTS OF THIS SPECIFICATION, SOFTWARE RELATED
// THERETO, CODE AND/OR PROGRAM RELATED THERETO UNLESS AGREED TO IN WRITING.
// THIS SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR PROGRAM RELATED
// THERETO IS BEING PROVIDED "AS IS", WITHOUT ANY WARRANTY OF ANY TYPE OR
// NATURE, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
// WARRANTY THAT THIS SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR
// PROGRAM RELATED THERETO IS FREE FROM DEFECTS.
//
// ASSUMPTION OF RISK: THE RISK OF ANY AND ALL LOSS, DAMAGE, OR UNSATISFACTORY
// PERFORMANCE OF THIS SPECIFICATION (RESPECTIVELY THE PRODUCTS MAKING USE OF
// IT IN PART OR AS A WHOLE), SOFTWARE RELATED THERETO, CODE AND/OR PROGRAM
// RELATED THERETO RESTS WITH YOU AS THE USER. TO THE MAXIMUM EXTENT PERMITTED
// BY LAW, NEITHER THE INTELLECTUAL PROPERTY OWNERS, COPYRIGHT HOLDERS AND
// CONTRIBUTORS, NOR ANY PERSON EITHER EXPRESSLY OR IMPLICITLY, MAKES ANY
// REPRESENTATION OR WARRANTY REGARDING THE APPROPRIOATENESS OF THE USE, OUTPUT,
// OR RESULTS OF THE USE OF THIS SPECIFICATION, SOFTWARE RELATED THERETO, CODE
// AND/OR PROGRAM RELATED THERETO IN TERMS OF ITS CORRECTNESS, ACCURACY,
// RELIABILITY, BEING CURRENT OR OTHERWISE. NOR DO THEY HAVE ANY OBLIGATION TO
// CORRECT ERRORS, MAKE CHANGES, SUPPORT THIS SPECIFICATION, SOFTWARE RELATED
// THERETO, CODE AND/OR PROGRAM RELATED THERETO, DISTRIBUTE UPDATES, OR PROVIDE
// NOTIFICATION OF ANY ERROR OR DEFECT, KNOWN OR UNKNOWN. IF YOU RELY UPON THIS
// SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR PROGRAM RELATED THERETO,
// YOU DO SO AT YOUR OWN RISK, AND YOU ASSUME THE RESPONSIBILITY FOR THE
// RESULTS. SHOULD THIS SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR
// PROGRAM RELATED THERETO PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL LOSSES,
// INCLUDING, BUT NOT LIMITED TO, ANY NECESSARY SERVICING, REPAIR OR CORRECTION
// OF ANY PROPERTY INVOLVED TO THE MAXIMUM EXTEND PERMITTED BY LAW.
//
// DISCLAIMER: IN NO EVENT, UNLESS REQUIRED BY LAW OR AGREED TO IN WRITING,
// SHALL THE INTELLECTUAL PROPERTY OWNERS, COPYRIGHT HOLDERS OR ANY PERSON BE
// LIABLE FOR ANY LOSS, EXPENSE OR DAMAGE, OF ANY TYPE OR NATURE ARISING OUT OF
// THE USE OF, OR INABILITY TO USE THIS SPECIFICATION, SOFTWARE RELATED THERETO,
// CODE AND/OR PROGRAM RELATED THERETO, INCLUDING, BUT NOT LIMITED TO, CLAIMS,
// SUITS OR CAUSES OF ACTION INVOLVING ALLEGED INFRINGEMENT OF COPYRIGHTS,
// PATENTS, TRADEMARKS, TRADE SECRETS, OR UNFAIR COMPETITION.
//
// INDEMNIFICATION: TO THE MAXIMUM EXTEND PERMITTED BY LAW YOU AGREE TO
// INDEMNIFY AND HOLD HARMLESS THE INTELLECTUAL PROPERTY OWNERS, COPYRIGHT
// HOLDERS AND CONTRIBUTORS, AND EMPLOYEES, AND ANY PERSON FROM AND AGAINST ALL
// CLAIMS, LIABILITIES, LOSSES, CAUSES OF ACTION, DAMAGES, JUDGMENTS, AND
// EXPENSES, INCLUDING THE REASONABLE COST OF ATTORNEYS' FEES AND COURT COSTS,
// FOR INJURIES OR DAMAGES TO THE PERSON OR PROPERTY OF THIRD PARTIES,
// INCLUDING, WITHOUT LIMITATIONS, CONSEQUENTIAL, DIRECT AND INDIRECT DAMAGES
// AND ANY ECONOMIC LOSSES, THAT ARISE OUT OF OR IN CONNECTION WITH YOUR USE,
// MODIFICATION, OR DISTRIBUTION OF THIS SPECIFICATION, SOFTWARE RELATED
// THERETO, CODE AND/OR PROGRAM RELATED THERETO, ITS OUTPUT, OR ANY ACCOMPANYING
// DOCUMENTATION.
//
// GOVERNING LAW: THE RELATIONSHIP BETWEEN YOU AND ROBERT BOSCH GMBH SHALL BE
// GOVERNED SOLELY BY THE LAWS OF THE FEDERAL REPUBLIC OF GERMANY. THE
// STIPULATIONS OF INTERNATIONAL CONVENTIONS REGARDING THE INTERNATIONAL SALE OF
// GOODS SHALL NOT BE APPLICABLE. THE EXCLUSIVE LEGAL VENUE SHALL BE
// DUESSELDORF, GERMANY.
//
// MANDATORY LAW SHALL BE UNAFFECTED BY THE FOREGOING PARAGRAPHS.
//
// INTELLECTUAL PROPERTY OWNERS/COPYRIGHT OWNERS/CONTRIBUTORS: ROBERT BOSCH
// GMBH, ROBERT BOSCH PLATZ 1, 70839 GERLINGEN, GERMANY AND ITS LICENSORS.
//
// ALL RIGHTS RESERVED.
//------------------------------------------------------------------------------
// Project Name : GTM - Generic Timer Module
//------------------------------------------------------------------------------
// Copyright   : Robert Bosch GmbH, Gerlingen, Germany
// Department  : AE/EIY
// Created     : 13.05.2011
//------------------------------------------------------------------------------
//:head-------------------------------------------------------------------------
//------------------------------------------------------------------------------
// Description     : virtual pointer of HAL
//============================================================================ 
// Important Notes : 
//============================================================================ 
#ifndef VIRTUAL_PTR_H_
#define VIRTUAL_PTR_H_

template <class IF, class T, class IDX=int>
class virtual_ptr {
	IDX pos;
	class PtrElement{
		friend class virtual_ptr;
		IDX pos;
		PtrElement(IDX _pos) {
			pos = _pos;
		}
	public:
		inline
		operator T() {
			return IF::read(pos);
		}
		inline
		void operator=(T rhs){
			IF::write(pos, rhs);
		}
        inline 
		virtual_ptr operator&() {
			return virtual_ptr(pos);
		} 
	};
public:
	// construction
	inline
	virtual_ptr(const IDX _pos=0) : pos(_pos) {
	}
	inline
	virtual_ptr(const void* _pos) : pos((IDX)_pos) {
	}
	// refering elements
	inline
	PtrElement operator [](const IDX idx) {
		return PtrElement(pos + idx * sizeof(T));
	}
	inline
	PtrElement operator*() {
		return PtrElement(pos);
	}
	// pointer assignment
	inline
	virtual_ptr operator=(const virtual_ptr& rhs) {
		pos = rhs.pos;
		return *this;
	}
	// pointer arithmetic (plus operators)
	inline
	virtual_ptr operator+(const IDX rhs) {
		return virtual_ptr(pos + rhs * sizeof(T));
	}
	inline
	virtual_ptr operator++() { // prefix
		pos += sizeof(T);
		return *this;
	}
	inline
	virtual_ptr operator++(int) { // postfix
		virtual_ptr lval(pos);
		pos += sizeof(T);
		return lval;
	}
	inline
	virtual_ptr operator+=(IDX rhs) { 
		pos += rhs * sizeof(T);
		return *this;
	}
	// pointer arithmetic (minus operators)
	inline
	virtual_ptr operator-(const IDX rhs) {
		return virtual_ptr(pos - rhs * sizeof(T));
	}
	inline
	virtual_ptr operator--() { // prefix
		pos -= sizeof(T);
		return *this;
	}
	inline
	virtual_ptr operator--(int) { // postfix
		virtual_ptr lval(pos);
		pos -= sizeof(T);
		return lval;
	}
	inline
	virtual_ptr operator-=(const IDX rhs) { 
		pos -= rhs * sizeof(T);
		return *this;
	}
	// pointer arithmetic (comparison operators)
	inline
	bool operator<(const virtual_ptr rhs) { 
		return pos < rhs.pos;
	}
	inline
	bool operator<=(const virtual_ptr rhs) { 
		return pos <= rhs.pos;
	}
	inline
	bool operator>(const virtual_ptr rhs) { 
		return pos > rhs.pos;
	}
	inline
	bool operator>=(const virtual_ptr rhs) { 
		return pos >= rhs.pos;
	}
	inline
	bool operator==(const virtual_ptr rhs) { 
		return pos == rhs.pos;
	}
	inline
	bool operator!=(const virtual_ptr rhs) { 
		return pos != rhs.pos;
	}
};

#endif

