--head:-------------------------------------------------------------------------
--:head-------------------------------------------------------------------------
--------------------------------------------------------------------------------
-- Description    : Creates 6 synchronous symmetrical PWM output signals
--                  Reload duty cycle values via FIFO
--                  Use channel 7 to count number of periods for update period
--============================================================================ 
-- Important Notes:
--============================================================================ 

--------------------------------------------------------------------------------------
-- WATCHDOG: 
CLK WATCHDOG 5001 us
--------------------------------------------------------------------------------------

#inc inc/aru_write_addr_map.cmd
#inc inc/chksig_labels.cmd
#inc inc_cfg/gtm_device_cfg.cmd
#inc inc/check_gtm_resources.cmd

AEI SET_RM_ACCESS_CYCLES 3
AEI SET_RM_WAIT_CYCLES 1

AEI print "GTM generation is " #GTM_GEN

-- end of testcase, if no ATOM instance available in current device
#if $(#psm_instances_cnt == 0)
   ALL SYNC ALL
   AEI print "=================================================================================="
   AEI print "Warning : PSM is not available in this device"
   AEI print "Warning : Test isn't executed"
   AEI print "=================================================================================="
   ALL SYNC ALL
   ALL QUIT
#endif

ALL IRQ MASK #900 1
ALL IRQ MASK #901 1
ALL IRQ MASK #902 1
ALL IRQ MASK #903 1
ALL IRQ MASK #904 1

CLK FREQ  100 MHz
CLK RESET  1  12
CLK WAIT deltime 100 ns

ALL SYNC ALL
AEI print "configure report level"
AEI print "======================================================="
DBI SET_RPT_LEVEL AEI   INFO
DBI SET_RPT_LEVEL SWI   INFO
--DBI SET_RPT_LEVEL CHK   INFO
--DBI SET_RPT_LEVEL atom0 DBG3
--DBI SET_RPT_LEVEL atom0_ch1 DBG3
--DBI SET_RPT_LEVEL atom0_ch2 DBG3
--DBI SET_RPT_LEVEL atom0_ch3 DBG3
--DBI SET_RPT_LEVEL atom0_ch4 DBG3
--DBI SET_RPT_LEVEL atom0_ch7 DBG3

ALL SYNC ALL

AEI print "Configure CHK IFS"
AEI print "======================================================="

-- check channel 0:
CHK ADD_INSTR 1 WSE #ATOM0_OUT0 RE         -- synchronize to first RE of channel 0 
CHK ADD_INSTR 1 MSE #ATOM0_OUT0 FE 25000   -- 25000ns high pulse
CHK ADD_INSTR 1 MSE #ATOM0_OUT0 RE 25000   -- 25000ns low pulse
CHK ADD_INSTR 1 JMT 30 -2      -- repeat 30 times  

-- check channel 1:
CHK ADD_INSTR 2 WSE #ATOM0_OUT1 RE        -- synchronize to first RE of channel 1 
#if $(#GTM_GEN == 1)
CHK ADD_INSTR 2 MSE #ATOM0_OUT1 FE 25000 -- 1. pulse high 25000 ns <- in 1st gen CN0 stops if CM0=0
#else
CHK ADD_INSTR 2 MSE #ATOM0_OUT1 FE 87450 -- 1. pulse high 87450 ns <- since 2nd gen CN0 counts if CM0=0
#endif
CHK ADD_INSTR 2 MSE #ATOM0_OUT1 RE 25000 --29100   later rising edge
CHK ADD_INSTR 2 MSE #ATOM0_OUT1 FE 25000 -- 2. pulse high 25000 ns
CHK ADD_INSTR 2 MSE #ATOM0_OUT1 RE 22950 --20900   later rising edge
CHK ADD_INSTR 2 MSE #ATOM0_OUT1 FE 29100 -- 3. pulse high 29100 ns
CHK ADD_INSTR 2 MSE #ATOM0_OUT1 RE 20900 --17000   later rising edge
CHK ADD_INSTR 2 MSE #ATOM0_OUT1 FE 29100 -- 4. pulse high 29100 ns
CHK ADD_INSTR 2 MSE #ATOM0_OUT1 RE 18850 --13100   later rising edge
CHK ADD_INSTR 2 MSE #ATOM0_OUT1 FE 33200 -- 5. pulse high 33200 ns
CHK ADD_INSTR 2 MSE #ATOM0_OUT1 RE 16800 --9900    later rising edge
CHK ADD_INSTR 2 MSE #ATOM0_OUT1 FE 33200 -- 6. pulse high 33200 ns

-- check channel 4:
CHK ADD_INSTR 3 WSE #ATOM0_OUT4 FE        -- synchronize to first FE of channel 4
#if $(#GTM_GEN == 1)
CHK ADD_INSTR 3 MSE #ATOM0_OUT4 RE 45700 -- 1. pulse high 45700 ns <- in 1st gen CN0 stops if CM0=0
#else
CHK ADD_INSTR 3 MSE #ATOM0_OUT4 RE 97800 -- 1. pulse high 97800 ns <- since 2nd gen CN0 counts if CM0=0
#endif
CHK ADD_INSTR 3 MSE #ATOM0_OUT4 FE  4300 --5200    later falling edge
CHK ADD_INSTR 3 MSE #ATOM0_OUT4 RE 45700 --43300   later rising edge
CHK ADD_INSTR 3 MSE #ATOM0_OUT4 FE  5500 --6700    later falling edge
CHK ADD_INSTR 3 MSE #ATOM0_OUT4 RE 43300 -- later rising edge
CHK ADD_INSTR 3 MSE #ATOM0_OUT4 FE  6700 --9850 later falling edge
CHK ADD_INSTR 3 MSE #ATOM0_OUT4 RE 43300 --37000   later rising edge
CHK ADD_INSTR 3 MSE #ATOM0_OUT4 FE  8150 --13000   later falling edge
CHK ADD_INSTR 3 MSE #ATOM0_OUT4 RE 40400 --37000   later rising edge
CHK ADD_INSTR 3 MSE #ATOM0_OUT4 FE  9600 -- later falling edge
CHK ADD_INSTR 3 MSE #ATOM0_OUT4 RE 40400 --37000   later rising edge

-- check channel 7:
CHK ADD_INSTR 4 WSE #ATOM0_OUT7 RE         -- synchronize to first RE of channel 7 
CHK ADD_INSTR 4 MSE #ATOM0_OUT7 FE 5000    -- 5000ns high pulse
CHK ADD_INSTR 4 MSE #ATOM0_OUT7 RE 95000   -- + 95000ns = 2*period channel 0
CHK ADD_INSTR 4 JMT 30 -2      -- repeat 30 times

CHK ENABLE 1 ON
CHK ENABLE 2 ON
CHK ENABLE 3 ON
CHK ENABLE 4 ON

ALL SYNC ALL

AEI print "start c-test 'atom_pmsm'"
SWI RUN_SW atom_pmsm

AEI print "Run simulation"
AEI print "======================================================="

AEI WAIT simtime 5000 us
ALL SYNC ALL

CHK ENABLE 1 OFF
CHK ENABLE 2 OFF
CHK ENABLE 3 OFF
CHK ENABLE 4 OFF

ALL SYNC ALL

AEI print "======================================================================"
AEI print "End of Test"
AEI print "======================================================================"
ALL SYNC ALL
ALL QUIT


#isr 900
AEI print "ATOM0-CH0/1 IRQ fired (irq-900)"
SWI RUN_ISR atom_pmsm_isr 900
#end

#isr 901
AEI print "ATOM0-CH2/3 IRQ fired (irq-901)"
SWI RUN_ISR atom_pmsm_isr 901
#end

#isr 902
AEI print "ATOM0-CH4/5 IRQ fired (irq-902)"
SWI RUN_ISR atom_pmsm_isr 902
#end

#isr 903
AEI print "ATOM0-CH6/7 IRQ fired (irq-903)"
SWI RUN_ISR atom_pmsm_isr 903
#end

#isr 904
AEI print "ATOM1-CH0/1 IRQ fired (irq-904)"
SWI RUN_ISR atom_pmsm_isr 903
#end

#inc inc/empty_isr_out_signals.cmd

-------------------------------------------------------------------------------
-- This is the footer of the file.
-------------------------------------------------------------------------------
