//head:-------------------------------------------------------------------------
//------------------------------------------------------------------------------
// $Id: sw_atom_flex_pwm_an012.cpp,v 1.2 2014/04/04 06:36:13 twagner Exp $
//------------------------------------------------------------------------------
// NO WARRANTIES: TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER THE
// INTELLECTUAL PROPERTY OWNERS, COPYRIGHT HOLDERS AND CONTRIBUTORS, NOR ANY
// PERSON, EITHER EXPRESSLY OR IMPLICITLY, WARRANTS ANY ASPECT OF THIS
// SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR PROGRAM RELATED THERETO,
// INCLUDING ANY OUTPUT OR RESULTS OF THIS SPECIFICATION, SOFTWARE RELATED
// THERETO, CODE AND/OR PROGRAM RELATED THERETO UNLESS AGREED TO IN WRITING.
// THIS SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR PROGRAM RELATED
// THERETO IS BEING PROVIDED "AS IS", WITHOUT ANY WARRANTY OF ANY TYPE OR
// NATURE, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
// WARRANTY THAT THIS SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR
// PROGRAM RELATED THERETO IS FREE FROM DEFECTS.
//
// ASSUMPTION OF RISK: THE RISK OF ANY AND ALL LOSS, DAMAGE, OR UNSATISFACTORY
// PERFORMANCE OF THIS SPECIFICATION (RESPECTIVELY THE PRODUCTS MAKING USE OF
// IT IN PART OR AS A WHOLE), SOFTWARE RELATED THERETO, CODE AND/OR PROGRAM
// RELATED THERETO RESTS WITH YOU AS THE USER. TO THE MAXIMUM EXTENT PERMITTED
// BY LAW, NEITHER THE INTELLECTUAL PROPERTY OWNERS, COPYRIGHT HOLDERS AND
// CONTRIBUTORS, NOR ANY PERSON EITHER EXPRESSLY OR IMPLICITLY, MAKES ANY
// REPRESENTATION OR WARRANTY REGARDING THE APPROPRIOATENESS OF THE USE, OUTPUT,
// OR RESULTS OF THE USE OF THIS SPECIFICATION, SOFTWARE RELATED THERETO, CODE
// AND/OR PROGRAM RELATED THERETO IN TERMS OF ITS CORRECTNESS, ACCURACY,
// RELIABILITY, BEING CURRENT OR OTHERWISE. NOR DO THEY HAVE ANY OBLIGATION TO
// CORRECT ERRORS, MAKE CHANGES, SUPPORT THIS SPECIFICATION, SOFTWARE RELATED
// THERETO, CODE AND/OR PROGRAM RELATED THERETO, DISTRIBUTE UPDATES, OR PROVIDE
// NOTIFICATION OF ANY ERROR OR DEFECT, KNOWN OR UNKNOWN. IF YOU RELY UPON THIS
// SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR PROGRAM RELATED THERETO,
// YOU DO SO AT YOUR OWN RISK, AND YOU ASSUME THE RESPONSIBILITY FOR THE
// RESULTS. SHOULD THIS SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR
// PROGRAM RELATED THERETO PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL LOSSES,
// INCLUDING, BUT NOT LIMITED TO, ANY NECESSARY SERVICING, REPAIR OR CORRECTION
// OF ANY PROPERTY INVOLVED TO THE MAXIMUM EXTEND PERMITTED BY LAW.
//
// DISCLAIMER: IN NO EVENT, UNLESS REQUIRED BY LAW OR AGREED TO IN WRITING,
// SHALL THE INTELLECTUAL PROPERTY OWNERS, COPYRIGHT HOLDERS OR ANY PERSON BE
// LIABLE FOR ANY LOSS, EXPENSE OR DAMAGE, OF ANY TYPE OR NATURE ARISING OUT OF
// THE USE OF, OR INABILITY TO USE THIS SPECIFICATION, SOFTWARE RELATED THERETO,
// CODE AND/OR PROGRAM RELATED THERETO, INCLUDING, BUT NOT LIMITED TO, CLAIMS,
// SUITS OR CAUSES OF ACTION INVOLVING ALLEGED INFRINGEMENT OF COPYRIGHTS,
// PATENTS, TRADEMARKS, TRADE SECRETS, OR UNFAIR COMPETITION.
//
// INDEMNIFICATION: TO THE MAXIMUM EXTEND PERMITTED BY LAW YOU AGREE TO
// INDEMNIFY AND HOLD HARMLESS THE INTELLECTUAL PROPERTY OWNERS, COPYRIGHT
// HOLDERS AND CONTRIBUTORS, AND EMPLOYEES, AND ANY PERSON FROM AND AGAINST ALL
// CLAIMS, LIABILITIES, LOSSES, CAUSES OF ACTION, DAMAGES, JUDGMENTS, AND
// EXPENSES, INCLUDING THE REASONABLE COST OF ATTORNEYS' FEES AND COURT COSTS,
// FOR INJURIES OR DAMAGES TO THE PERSON OR PROPERTY OF THIRD PARTIES,
// INCLUDING, WITHOUT LIMITATIONS, CONSEQUENTIAL, DIRECT AND INDIRECT DAMAGES
// AND ANY ECONOMIC LOSSES, THAT ARISE OUT OF OR IN CONNECTION WITH YOUR USE,
// MODIFICATION, OR DISTRIBUTION OF THIS SPECIFICATION, SOFTWARE RELATED
// THERETO, CODE AND/OR PROGRAM RELATED THERETO, ITS OUTPUT, OR ANY ACCOMPANYING
// DOCUMENTATION.
//
// GOVERNING LAW: THE RELATIONSHIP BETWEEN YOU AND ROBERT BOSCH GMBH SHALL BE
// GOVERNED SOLELY BY THE LAWS OF THE FEDERAL REPUBLIC OF GERMANY. THE
// STIPULATIONS OF INTERNATIONAL CONVENTIONS REGARDING THE INTERNATIONAL SALE OF
// GOODS SHALL NOT BE APPLICABLE. THE EXCLUSIVE LEGAL VENUE SHALL BE
// DUESSELDORF, GERMANY.
//
// MANDATORY LAW SHALL BE UNAFFECTED BY THE FOREGOING PARAGRAPHS.
//
// INTELLECTUAL PROPERTY OWNERS/COPYRIGHT OWNERS/CONTRIBUTORS: ROBERT BOSCH
// GMBH, ROBERT BOSCH PLATZ 1, 70839 GERLINGEN, GERMANY AND ITS LICENSORS.
//
// ALL RIGHTS RESERVED.
//------------------------------------------------------------------------------
// Project Name : GTM - Generic Timer Module
//------------------------------------------------------------------------------
// Copyright   : Robert Bosch GmbH, Gerlingen, Germany
// Department  : AE/EIY
// Created     : 30.01.2014
//------------------------------------------------------------------------------
//:head-------------------------------------------------------------------------
//------------------------------------------------------------------------------
// Description      :   sw_atom_flex_pwm_an012
//                      implements 
//                      Application Note AN012
//                      ATOM Flexible PWM generation
//============================================================================ 
// Important Notes : 
//============================================================================ 

#include "gtm.h"
#include "gtm_device_cfg.h"
#include "gtm_defs.h"
#include "functions.h"
#include <cstdlib>

//------------------------------------------------------------------------------
// init functions

void init_cmu_an012(void)
{

    // setup global clock divider
    CMU_GCLK_NUM = 0xFFFFFF;
    CMU_GCLK_DEN = 0xFFFFFF;

    // setup the CMU_CLKx prescalers
    CMU_CLK_0_CTRL = 0x9; // divide SYS_CLK by 10
	
	// enable clock prescalers (CMU_CLK0 only)
    CMU_CLK_EN = 0x2;

}  //init_cmu_an012()


void init_tbu_an012(void)
{

    // setup TBU channel 0
    TBU_CH0_CTRL = 0x0; // no LOW_RES, chose CMU_CLK0

    // enable TBU channel 0
    TBU_CHEN = 0x2;	
	
}  //init_tbu_an012()


void init_psm_an012(void)
{

    // setup FIFO not needed for this application
    // setup F2A channels 0 and 1 to provide 24 bit of data on the upper 
    // ARU word
    F2A0_CH0_STR_CFG = 0x50000; // transfer high word, FIFO -> ARU
    F2A0_CH1_STR_CFG = 0x50000; // transfer high word, FIFO -> ARU

    // enable F2A channels 0 and 1
    F2A0_ENABLE = 0xA;
	
}  //init_psm_an012()


void init_atom_an012(void)
{

    #define PWM_PERIOD 100 			//define PWM period to 10us (100*100ns)
    
    // setup ATOM channels 0 in SOMP
    ATOM0_CH0_RDADDR = F2A0_WRADDR0;    	// use data from PSM0 channel 0
    ATOM0_CH0_CN0 = 99; 			// start immediately with PWM
    ATOM0_CH0_CM0 = PWM_PERIOD; 		// set PWM period
    ATOM0_CH0_SR0 = PWM_PERIOD; 		// set PWM period
    ATOM0_CH0_CTRL = 0x82A; 			// SL=1, CMU_CLK0, ARU_EN=1,
						// load ARU high word, SOMP

    // setup ATOM channel 1 in SOMP
    ATOM0_CH1_RDADDR = F2A0_WRADDR1; 	        // use data from PSM0 channel 1
    ATOM0_CH1_CN0 = 49; 			// start 50% later than ch0 PWM
    ATOM0_CH1_CM0 = PWM_PERIOD; 		// set PWM period
    ATOM0_CH1_SR0 = PWM_PERIOD; 		// set PWM period
    ATOM0_CH1_CTRL = 0x82A; 			// SL=1, CMU_CLK0, ARU_EN=1,
						// load ARU high word, SOMP

    // setup ATOM channel 7 in SOMC and enable CCU1TC interrupt
    ATOM0_CH7_CTRL = 0x921; 			// SL=1, ACB42=100, ARU_EN=0, SOMC   
    ATOM0_CH7_IRQ_EN = 0x2; 			// enable CCU1TC IRQ
    ATOM0_CH7_CM0 = 1280; 			// rising edge 108us after 1st PWM
    ATOM0_CH7_CM1 = 1281; 			// falling edge 100ns later
	
    // alternatie: setup ATOM channel 6 in SOMC with different settings
    ATOM0_CH6_CTRL = 0x121; 			// SL=0, ACB42=100, ARU_EN=0, SOMC   
    ATOM0_CH6_CM0 = 1280; 			// rising edge 108us after 1st PWM
    ATOM0_CH6_CM1 = 1281; 			// falling edge 100ns later
	
    // setup AGC
    ATOM0_AGC_FUPD_CTRL = 0x0000A;              // force update on ch0 and 1
    ATOM0_AGC_GLB_CTRL  = 0xA0001; 		// enable update mechanism for ch. 1, 0
                                                // &  
                                                // host trigger to trigger forced update

    ATOM0_AGC_ENDIS_CTRL = 0xA00A;  	        // enable ch. 7, 6, 1, 0
	
    ATOM0_AGC_ACT_TB = 0x010000C8;  	        // set up time base cmp for value 200
     
    ATOM0_AGC_OUTEN_CTRL = 0xA00A;              // enable ATOM output channel 7, 6, 1, 0
	
}  //init_atom_an012()


//------------------------------------------------------------------------------
// main thread:
int sw_atom_flex_pwm_an012(void)
{
    if ((PSM_INSTANCES  > 0) &&
		(ATOM_INSTANCES > 0) )
	{
		// initialize and enable all modules
		init_cmu_an012();
		init_tbu_an012();
		init_psm_an012();
		init_atom_an012();

		// provide initial PWM duty cycles (50%): fill PSM channels 
		for (int i=0; i<10; i++) { 
			AFD0_CH0_BUF_ACC = 50;
			AFD0_CH1_BUF_ACC = 50;
		} // for	
	} // if
	
    return 0;

} // sw_atom_flex_pwm_an012()


//------------------------------------------------------------------------------
// ISR

int sw_atom_flex_pwm_an012_isr(int isr_nr)
{

    int mp = 0; // matching time of compare
    int dummy = 0;

    // refill PSM with new PWM duty cycles
    for (int i=0; i<11; i++) {
        AFD0_CH0_BUF_ACC = i*10; 	// generate PWM from 0% to 100%
        AFD0_CH1_BUF_ACC = i*10;	// generate PWM from 0% to 100%
    }

    // clear IRQ and set up new capture/compare values for channel 7
    ATOM0_CH7_IRQ_NOTIFY = 0x2; 	// clear IRQ notify bit
	
    mp = ATOM0_CH7_SR0; 		// read match time, 
					// enables cap/com again!
    dummy = ATOM0_CH6_SR0; 		// dummy read match time, 
					// enables cap/com again!
									
    ATOM0_CH7_CM0 = mp + 1079; 		// set up new match event
    ATOM0_CH7_CM1 = mp + 1080; 		// set up new match event
    ATOM0_CH6_CM0 = mp + 1079;   	// set up new match event
    ATOM0_CH6_CM1 = mp + 1080; 		// set up new match event
	
    return 0;

} //atom_flex_pwm_an12_isr()









