//head:-------------------------------------------------------------------------
//------------------------------------------------------------------------------
// $Id: sw_dpll_an011.cpp,v 1.4 2013/09/20 11:19:34 opfitzer Exp $
//------------------------------------------------------------------------------
// NO WARRANTIES: TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER THE
// INTELLECTUAL PROPERTY OWNERS, COPYRIGHT HOLDERS AND CONTRIBUTORS, NOR ANY
// PERSON, EITHER EXPRESSLY OR IMPLICITLY, WARRANTS ANY ASPECT OF THIS
// SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR PROGRAM RELATED THERETO,
// INCLUDING ANY OUTPUT OR RESULTS OF THIS SPECIFICATION, SOFTWARE RELATED
// THERETO, CODE AND/OR PROGRAM RELATED THERETO UNLESS AGREED TO IN WRITING.
// THIS SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR PROGRAM RELATED
// THERETO IS BEING PROVIDED "AS IS", WITHOUT ANY WARRANTY OF ANY TYPE OR
// NATURE, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
// WARRANTY THAT THIS SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR
// PROGRAM RELATED THERETO IS FREE FROM DEFECTS.
//
// ASSUMPTION OF RISK: THE RISK OF ANY AND ALL LOSS, DAMAGE, OR UNSATISFACTORY
// PERFORMANCE OF THIS SPECIFICATION (RESPECTIVELY THE PRODUCTS MAKING USE OF
// IT IN PART OR AS A WHOLE), SOFTWARE RELATED THERETO, CODE AND/OR PROGRAM
// RELATED THERETO RESTS WITH YOU AS THE USER. TO THE MAXIMUM EXTENT PERMITTED
// BY LAW, NEITHER THE INTELLECTUAL PROPERTY OWNERS, COPYRIGHT HOLDERS AND
// CONTRIBUTORS, NOR ANY PERSON EITHER EXPRESSLY OR IMPLICITLY, MAKES ANY
// REPRESENTATION OR WARRANTY REGARDING THE APPROPRIOATENESS OF THE USE, OUTPUT,
// OR RESULTS OF THE USE OF THIS SPECIFICATION, SOFTWARE RELATED THERETO, CODE
// AND/OR PROGRAM RELATED THERETO IN TERMS OF ITS CORRECTNESS, ACCURACY,
// RELIABILITY, BEING CURRENT OR OTHERWISE. NOR DO THEY HAVE ANY OBLIGATION TO
// CORRECT ERRORS, MAKE CHANGES, SUPPORT THIS SPECIFICATION, SOFTWARE RELATED
// THERETO, CODE AND/OR PROGRAM RELATED THERETO, DISTRIBUTE UPDATES, OR PROVIDE
// NOTIFICATION OF ANY ERROR OR DEFECT, KNOWN OR UNKNOWN. IF YOU RELY UPON THIS
// SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR PROGRAM RELATED THERETO,
// YOU DO SO AT YOUR OWN RISK, AND YOU ASSUME THE RESPONSIBILITY FOR THE
// RESULTS. SHOULD THIS SPECIFICATION, SOFTWARE RELATED THERETO, CODE AND/OR
// PROGRAM RELATED THERETO PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL LOSSES,
// INCLUDING, BUT NOT LIMITED TO, ANY NECESSARY SERVICING, REPAIR OR CORRECTION
// OF ANY PROPERTY INVOLVED TO THE MAXIMUM EXTEND PERMITTED BY LAW.
//
// DISCLAIMER: IN NO EVENT, UNLESS REQUIRED BY LAW OR AGREED TO IN WRITING,
// SHALL THE INTELLECTUAL PROPERTY OWNERS, COPYRIGHT HOLDERS OR ANY PERSON BE
// LIABLE FOR ANY LOSS, EXPENSE OR DAMAGE, OF ANY TYPE OR NATURE ARISING OUT OF
// THE USE OF, OR INABILITY TO USE THIS SPECIFICATION, SOFTWARE RELATED THERETO,
// CODE AND/OR PROGRAM RELATED THERETO, INCLUDING, BUT NOT LIMITED TO, CLAIMS,
// SUITS OR CAUSES OF ACTION INVOLVING ALLEGED INFRINGEMENT OF COPYRIGHTS,
// PATENTS, TRADEMARKS, TRADE SECRETS, OR UNFAIR COMPETITION.
//
// INDEMNIFICATION: TO THE MAXIMUM EXTEND PERMITTED BY LAW YOU AGREE TO
// INDEMNIFY AND HOLD HARMLESS THE INTELLECTUAL PROPERTY OWNERS, COPYRIGHT
// HOLDERS AND CONTRIBUTORS, AND EMPLOYEES, AND ANY PERSON FROM AND AGAINST ALL
// CLAIMS, LIABILITIES, LOSSES, CAUSES OF ACTION, DAMAGES, JUDGMENTS, AND
// EXPENSES, INCLUDING THE REASONABLE COST OF ATTORNEYS' FEES AND COURT COSTS,
// FOR INJURIES OR DAMAGES TO THE PERSON OR PROPERTY OF THIRD PARTIES,
// INCLUDING, WITHOUT LIMITATIONS, CONSEQUENTIAL, DIRECT AND INDIRECT DAMAGES
// AND ANY ECONOMIC LOSSES, THAT ARISE OUT OF OR IN CONNECTION WITH YOUR USE,
// MODIFICATION, OR DISTRIBUTION OF THIS SPECIFICATION, SOFTWARE RELATED
// THERETO, CODE AND/OR PROGRAM RELATED THERETO, ITS OUTPUT, OR ANY ACCOMPANYING
// DOCUMENTATION.
//
// GOVERNING LAW: THE RELATIONSHIP BETWEEN YOU AND ROBERT BOSCH GMBH SHALL BE
// GOVERNED SOLELY BY THE LAWS OF THE FEDERAL REPUBLIC OF GERMANY. THE
// STIPULATIONS OF INTERNATIONAL CONVENTIONS REGARDING THE INTERNATIONAL SALE OF
// GOODS SHALL NOT BE APPLICABLE. THE EXCLUSIVE LEGAL VENUE SHALL BE
// DUESSELDORF, GERMANY.
//
// MANDATORY LAW SHALL BE UNAFFECTED BY THE FOREGOING PARAGRAPHS.
//
// INTELLECTUAL PROPERTY OWNERS/COPYRIGHT OWNERS/CONTRIBUTORS: ROBERT BOSCH
// GMBH, ROBERT BOSCH PLATZ 1, 70839 GERLINGEN, GERMANY AND ITS LICENSORS.
//
// ALL RIGHTS RESERVED.
//------------------------------------------------------------------------------
// Project Name : GTM - Generic Timer Module
//------------------------------------------------------------------------------
// Copyright   : Robert Bosch GmbH, Gerlingen, Germany
// Department  : AE/EIY
// Created     : 10.08.2011
//------------------------------------------------------------------------------
//:head-------------------------------------------------------------------------
//------------------------------------------------------------------------------
// Description     : Application note AN011: DPLL SUB_INC Generation
//============================================================================ 
// Important Notes : 
//============================================================================ 

#include <iostream>
#include "platform.h"
#include "gtm.h"         // includes also register_map.h
#include "functions.h"
#include "gtm_device_cfg.h"
#include "dpll_def.h"
#include "dpll_functions.h"
#include "loopback.h"

//#include <math.h>
//#include <cstdlib>
//#include "aru_write_addr_map.h"

using namespace std;

unsigned int iter    = 0;  // tooth detection iterations
unsigned int oldTS   = 0;  // remember old time stamp at _falling_ edge
unsigned int newTS   = 0;  // remember new time stamp at _falling_ edge
unsigned int diffTS = 0;  // diffTS = newTS - oldTS
unsigned int oldDiff = 0;  // time base difference one tooth before

extern unsigned int dut_is_rm;

//------------------------------------------------------------------------------
// setup of testbench
// PSM channel 0 to provide TRIGGER input
// ATOM0 channel 7 to generate PWM representing TRIGGER signal
//------------------------------------------------------------------------------
void tb_setup(void)
{
  int i = 0;
  // setup PSM channel 0
  FIFO0_CH0_CTRL		= 0x1;			// PSM operates in Ring Buffer Mode
  F2A0_CH0_STR_CFG		= 0x60000;		// transfer both words to ARU
  for (i=0; i<57; i++) {
	AFD0_CH0_BUF_ACC = 3200;			// 57 times normal tooth
	AFD0_CH0_BUF_ACC = 1600;			// 50% duty cycle
  }
  AFD0_CH0_BUF_ACC		= 9600;			// 58 th tooth + 2 tooth gap
  AFD0_CH0_BUF_ACC		= 1600;

  F2A0_ENABLE = 0x2;					// enable channel 0

  // configure ATOM0 channel 7 output
  ATOM0_CH7_RDADDR		= 0x051;		// get data from PSM channel0
  ATOM0_CH7_CTRL		= 0xA;			// ATOM channel 7 in SOMP, ARU enabled

  ATOM0_AGC_GLB_CTRL	= 0x80000000;	// enable update for shadow registers
  ATOM0_AGC_OUTEN_STAT	= 0x8000;		// enable channel 7 output
  ATOM0_AGC_ENDIS_STAT	= 0x8000;		// enable channel 7

  // configure ATOM0 channel 0 sensitive for micro ticks
  ATOM0_CH0_CTRL		= 0x000008D1;   // wait for the first match event and set to low
  ATOM0_CH0_CM1			= 600;
  ATOM0_CH0_IRQ_EN		= 0x2;			// enable CCU1 match interrupt

  ATOM0_AGC_OUTEN_STAT	= 0x0002;		// enable channel 0 output
  ATOM0_AGC_ENDIS_STAT	= 0x0002;		// enable channel 0

}

//------------------------------------------------------------------------------
// CMU Submodule setup
//------------------------------------------------------------------------------
void init_cmu(tCmu_param cParam, unsigned int en_msk)
{
	// setup global clock divider
	CMU_GCLK_NUM 	= cParam.gclk_num;
	CMU_GCLK_DEN 	= cParam.gclk_den;

	// setup the CMU_CLKx prescalers
	CMU_CLK_0_CTRL	= cParam.clk_0;
	CMU_CLK_1_CTRL	= cParam.clk_1;
	CMU_CLK_2_CTRL	= cParam.clk_2;
	CMU_CLK_3_CTRL	= cParam.clk_3;
	CMU_CLK_4_CTRL	= cParam.clk_4;
	CMU_CLK_5_CTRL	= cParam.clk_5;
	CMU_CLK_6_CTRL	= cParam.clk_6;
	CMU_CLK_7_CTRL	= cParam.clk_7;

	// enable the clock prescalers
	CMU_CLK_EN		= en_msk;
}

//------------------------------------------------------------------------------
// CMU Submodule setup
//------------------------------------------------------------------------------
void init_tbu(void)
{
	// setup TBU channel 0
	TBU_CH0_CTRL = 0x0;	// no LOW_RES, choose CMU_CLK0
	// setup TBU channel 1
	TBU_CH1_CTRL = 0x1;	// Up/Down counter mode; sub_inc1c is chosen

	// enable TBU channels 0 and 1
	TBU_CHEN	 = 0xA;
}

//------------------------------------------------------------------------------
// DPLL RAM2c initialization
//------------------------------------------------------------------------------
void init_dpll(void)
{
	unsigned int ram_ini_v = 0;
	unsigned int i = 0;
	gtm_ptr p;

	// initialize RAM; make sure that no RAM initialization takes place
	// in parallel
	ram_ini_v = DPLL_RAM_INI;
	while(ram_ini_v) { // we have to wait until RAM initialization ends
		cout << "waiting for RAM2 initialization to end: " << hex << ram_ini_v << endl;
	  ram_ini_v = DPLL_RAM_INI;
	}
	cout << "RAM initialization ended!" << endl;

    DPLL_OSW = 0x0;     // set RAM2 regions to hold 128 words each

	p = &DPLL_RR2;
	p = p + (0x00000400/4);
	for (i=0; i<57; i++){  	// file profile for regular tooth
	  p[i] = 0x10000;	// first HALF SCALE
	}
	p[57] = 0x30000;   // 58th is special one!
	p[58] = 0x12000;     // generate TINT0 interrupt at the 59th tooth in FULL_SCALE
	for (i=59; i<115; i++){  	// file profile for regular tooth
	  p[i] = 0x10000;	// second HALF SCALE
	}
	p[115] = 0x30000;
	
	DPLL_IRQ_EN = 0x00040000;  // enable TINT0 interrupt

    DPLL_TOV = 0x780;

	// Configure DPLL control registers
	DPLL_CTRL_0 = 0x403B0257;
    DPLL_CTRL_1 = 0x80020000;  // configure TRIGGER input charact. first
	DPLL_CTRL_1 = 0x80020012;  // now enable the DPLL
}



//------------------------------------------------------------------------------
// main thread:

int dpll_an011() {

  // required declaration for HAL
  gtm_ptr p;
  // CMU configuration structure
  tCmu_param theCmuConfig;

  // testbench setup
  tb_setup();
  
  // determine underlying hardware
  dut_is_rm   = GTM_REV;
  dut_is_rm >>= 20;
  if (dut_is_rm == 0x100)
    dut_is_rm = 1;
  else
    dut_is_rm = 0;

  theCmuConfig.gclk_num = 0xFFFFFF;		// define 100 MHz clock
  theCmuConfig.gclk_den = 0xFFFFFF;

  theCmuConfig.clk_0	= 0x4;			// define 20 MHz clock
  theCmuConfig.clk_1	= 0;			
  theCmuConfig.clk_2	= 0;			
  theCmuConfig.clk_3	= 0;			
  theCmuConfig.clk_4	= 0;			
  theCmuConfig.clk_5	= 0;			
  theCmuConfig.clk_6	= 0;			
  theCmuConfig.clk_7	= 0;			

  cout << "configure CMU" << endl;
  cout << "========================================================" << endl;
  init_cmu(theCmuConfig, 0x2);	// configure and enable CMU_CLK0

  cout << "Configure TBU" << endl;
  cout << "=======================================================" << endl;
  init_tbu();

  cout << "Configure TIM0 channel 0" << endl;
  cout << "=======================================================" << endl;
  TIM0_CH0_IRQ_EN	=   0x1;		// enable NEWVAL IRQ
  TIM0_CH0_CTRL     =   0x00000040; 
  TIM0_CH0_CTRL		=   0x00004045; // for TRIGGER 

  cout << "Configure DPLL" << endl;
  cout << "=======================================================" << endl;
  init_dpll();

  return 0;
}

//=============================================================================
// ISR:

int dpll_an011_isr(int number) {
	
  unsigned int actTS  = 0;	// actual sampled time stamp at TIM0 channel 0

  cout << "-----> ISR call of interrupt " << dec << number << endl; 

  switch (number) {

	case 118: 
	  cout << "DPLL TINT0 interrupt" << endl;
	  // check DPLL TRIGGER lock1 is set
	  DPLL_IRQ_NOTIFY = 0x00040000;
	  actTS = DPLL_STATUS;  // use actTS as tmp variable
	  actTS &= 0x40000000;
	  if (actTS)
		cout << "DPLL locked." << endl;
	  else
		cout << "ERROR: DPLL not locked yet!" << endl;
	  break;

	case 200: 
	  cout << "TIM0 Channel 0 interrupt" << endl;
	  // disable NOTIFY bit
	  TIM0_CH0_IRQ_NOTIFY = 0x1;
	  // tooth starts with falling edge
	  actTS = TIM0_CH0_GPR0;
	  cout << "Actual time stamp is: " << hex << actTS << endl;
	  if (!(actTS & 0x01000000)) { // falling edge detected
		// save old time stamp and tooth charateristic
		oldTS   = newTS;
		oldDiff = diffTS;
		// determine new time stamp
	    newTS = actTS & 0xFFFFFF;

	    // gap detection
	    iter++;
	    // wait for 
	    if (iter > 1) { // we're at the second tooth
		  diffTS = newTS - oldTS;
		  cout << "Time stamp difference: " << dec << diffTS << endl;
		  if (oldDiff >= 2*diffTS) {
			cout << "Gap detected two tooth before!" << endl;
			// synchronize DPLL
			if (dut_is_rm)
			  DPLL_APT_2C = (57+3)*4;	// use first gap position + 3 tooth and word addressing
			else
			  DPLL_APT_2C = (57+2)*4;	// use first gap position + 2 tooth and word addressing
			// enable DPLL sub_inc1c generation
			DPLL_CTRL_1 = 0x80020032;
			// disable TIM0 channel 0 NEWVAL interrupt
			TIM0_CH0_IRQ_EN = 0x0;
		  }
		}
	  }
      break;

	case 900: 
	  cout << "ATOM0 Channel 0 interrupt" << endl;
	  // disable NOTIFY bit
	  ATOM0_CH0_IRQ_NOTIFY = 0x2;
	  // calculate next trigger events
	  ATOM0_CH0_CTRL = 0x00000951;  // configure ACB42 to serve last
      actTS = ATOM0_CH0_SR1;
	  ATOM0_CH0_CM0 = actTS + 300;  // generate TRIGGER signal on ATOM0 channel 0
	  ATOM0_CH0_CM1 = actTS + 600;
	  break;
	default:
		  break;
  }

  return 0;
}
