/*
 *  (C) Robert Bosch GmbH 2008. All rights reserved, also regarding any
 *  disposal, exploitation, reproduction, editing, distribution, as
 *  well as in the event of application for industrial property rights.
 *
 *  Contact: AE/NE1-IFS <vAENE1.IFS@de.bosch.com>
 *
 * Disclaimer:
 * This Software is licensed under the following conditions. You may 
 * not use this file except in compliance with the following conditions:
 *
 * Robert Bosch GmbH hereby grants to licensee the non-exclusive and 
 * non-transferable right to use the software and accompanying 
 * documentation for the sole purpose of testing designs delivered by 
 * Robert Bosch GmbH to licensee.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY WARRANTY,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND 
 * NONINFRINGEMENT. IN NO EVENT SHALL ROBERT BOSCH GMBH BE LIABLE FOR
 * ANY DIRECT OR INDIRECT DAMAGES, INJURY OR OTHER LIABILITY, ARISING
 * OUT OR IN CONNECTION WITH THE SOFTWARE. ROBERT BOSCH GMBH SHALL
 * NOT BE LIABLE FOR THE ACCURACY OF THE DESIGNS TESTED WITH THE AID
 * OF THE SOFTWARE.
*/

#ifndef IFS_COMMAND_POINTER_H
#define IFS_COMMAND_POINTER_H

#include <list>
#include <string>

#include "IFS_CommandPointerBase.h"

namespace IFS
{
  /// A module-specific command pointer
  /** Since pointers to members are class dependent, each IFS module needs its
    * own command pointer class. Therefore we define this template class.
    * This class is not directly used by the IFS module's writer, but it is
    * hidden behind the macro IFS_REGISTER_COMMAND.
    * The controller only sees the module-independent base class
    * IFS_CommandPointerBase.
    * Basically this class stores an object pointer and a member function pointer.
    *
    */
  template< class ClassType >
  class IFS_CommandPointer : public IFS_CommandPointerBase
  {
    public:
      
      /// We define a typedef for the pointer to the member function for better readability
      typedef
        void (ClassType::*CommandMethodListPointer)( std::list<std::string> parameterList );
      
      /// Constructor for command-methods (with a list of string-parameters)
      IFS_CommandPointer(
        ClassType*               initialModulePointer,
        CommandMethodListPointer initialMethodPointer );

      /// \brief Call the command method of the IFS module 
      ///        (implements IFS_CommandPointerBase::callMethod)
      void callMethod( std::list<std::string> parameterList );
  
    private:
      
      /// A pointer to the IFS module providing the method
      ClassType*                 m_ModulePointer;

      /// A pointer to the member function which is to be called within the given IFS module.
      CommandMethodListPointer   m_MethodListPointer;
  };

}

#include "IFS_CommandPointer.icc"

#endif

