/*
 *  (C) Robert Bosch GmbH 2008. All rights reserved, also regarding any
 *  disposal, exploitation, reproduction, editing, distribution, as
 *  well as in the event of application for industrial property rights.
 *
 *  Contact: AE/NE1-IFS <vAENE1.IFS@de.bosch.com>
 *
 * Disclaimer:
 * This Software is licensed under the following conditions. You may 
 * not use this file except in compliance with the following conditions:
 *
 * Robert Bosch GmbH hereby grants to licensee the non-exclusive and 
 * non-transferable right to use the software and accompanying 
 * documentation for the sole purpose of testing designs delivered by 
 * Robert Bosch GmbH to licensee.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY WARRANTY,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND 
 * NONINFRINGEMENT. IN NO EVENT SHALL ROBERT BOSCH GMBH BE LIABLE FOR
 * ANY DIRECT OR INDIRECT DAMAGES, INJURY OR OTHER LIABILITY, ARISING
 * OUT OR IN CONNECTION WITH THE SOFTWARE. ROBERT BOSCH GMBH SHALL
 * NOT BE LIABLE FOR THE ACCURACY OF THE DESIGNS TESTED WITH THE AID
 * OF THE SOFTWARE.
*/

#ifndef IFS_COMMAND_POINTER_BASE_H
#define IFS_COMMAND_POINTER_BASE_H

#include <list>
#include <string>

namespace IFS
{
  /// The base class for the pointers to the IFS modules' command-methods.
  /** This class is used as the polymorphic base (needed by the controller)
    * which holds pointers to the methods to be called for registered 
    * commands of IFS-modules.
    */
  class IFS_CommandPointerBase
  {
    public:

      virtual ~IFS_CommandPointerBase() {}

      /// Call the actual command method (of the IFS module) with the given parameters 
      virtual void callMethod( std::list<std::string> parameterList ) = 0;
  };

}

#endif

