/*
 *  (C) Robert Bosch GmbH 2008. All rights reserved, also regarding any
 *  disposal, exploitation, reproduction, editing, distribution, as
 *  well as in the event of application for industrial property rights.
 *
 *  Contact: AE/NE1-IFS <vAENE1.IFS@de.bosch.com>
 *
 * Disclaimer:
 * This Software is licensed under the following conditions. You may 
 * not use this file except in compliance with the following conditions:
 *
 * Robert Bosch GmbH hereby grants to licensee the non-exclusive and 
 * non-transferable right to use the software and accompanying 
 * documentation for the sole purpose of testing designs delivered by 
 * Robert Bosch GmbH to licensee.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY WARRANTY,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND 
 * NONINFRINGEMENT. IN NO EVENT SHALL ROBERT BOSCH GMBH BE LIABLE FOR
 * ANY DIRECT OR INDIRECT DAMAGES, INJURY OR OTHER LIABILITY, ARISING
 * OUT OR IN CONNECTION WITH THE SOFTWARE. ROBERT BOSCH GMBH SHALL
 * NOT BE LIABLE FOR THE ACCURACY OF THE DESIGNS TESTED WITH THE AID
 * OF THE SOFTWARE.
*/

#ifndef IFS_COVERAGEBASE_H
#define IFS_COVERAGEBASE_H

#include <map>
#include <vector>
#include <sstream>
#include <stdarg.h>
#include <string>
#include <systemc.h>
#include "IFS_ErrorMessage.h"
#include "IFS_OutputHandler.h"

namespace IFS
{
  class IFS_CoverageBase {
    
    public:
      /// \brief
      /// Check the coverage status of parameter signals.
      /// If all signals are "covered", simulation will be stopped.
      /// \param count = number of following arguments
      /// \param last_named_arg = indicates the start of a variable parameter list
      void coverage_status(unsigned int count, bool last_named_arg,...);
    
      /// \brief
      /// function returns the actual value of a specific coverage signal
      /// if the signal is not registered, an exception will be thrown
      /// \param signalName = coverage signal name
      /// \return Returns actual value of a registered coverage signal
      bool getCoverageSignal(const std::string& signalName) throw (IFS_ErrorMessage);

      /// \brief
      /// store pointers to coverage signals
      /// \param signalName = coverage signal name
      /// \param signal = pointer to coverage signal
      void registerCoverageSignal(const std::string& signalName, sc_signal<bool> * signal) throw (IFS_ErrorMessage);

      /// \brief
      /// Check the coverage status of registered coverage signals.
      /// If all signals are "covered", simulation will be stopped.
      void coverage_status();
    
    private:

      /// store registered coverage signals (necessary to pass coverage information to cmd-file)
      std::map<const std::string,const sc_signal<bool>* > m_CoverageSignalMapping;
  };
  
  /// global instance of the output handler
  extern IFS_CoverageBase m_CoverageBase;
}

#endif /*IFS_COVERAGEBASE_H*/

