/*
 *  (C) Robert Bosch GmbH 2008. All rights reserved, also regarding any
 *  disposal, exploitation, reproduction, editing, distribution, as
 *  well as in the event of application for industrial property rights.
 *
 *  Contact: AE/NE1-IFS <vAENE1.IFS@de.bosch.com>
 *
 * Disclaimer:
 * This Software is licensed under the following conditions. You may 
 * not use this file except in compliance with the following conditions:
 *
 * Robert Bosch GmbH hereby grants to licensee the non-exclusive and 
 * non-transferable right to use the software and accompanying 
 * documentation for the sole purpose of testing designs delivered by 
 * Robert Bosch GmbH to licensee.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY WARRANTY,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND 
 * NONINFRINGEMENT. IN NO EVENT SHALL ROBERT BOSCH GMBH BE LIABLE FOR
 * ANY DIRECT OR INDIRECT DAMAGES, INJURY OR OTHER LIABILITY, ARISING
 * OUT OR IN CONNECTION WITH THE SOFTWARE. ROBERT BOSCH GMBH SHALL
 * NOT BE LIABLE FOR THE ACCURACY OF THE DESIGNS TESTED WITH THE AID
 * OF THE SOFTWARE.
*/

#ifndef IFS_SCRIPT_H
#define IFS_SCRIPT_H

#include <list>
#include <string>

namespace IFS
{

  /// A preprocessed line from the script
  struct SourceLine
  {
    /// line number (first one is 1)
    unsigned int m_LineNumber;
    /// the line itself
    std::string  m_LineContent;
    /// the file from which the line originates (\#inc)
    std::string  m_FileName;
  };

  class IFS_Control;

  /// An IFS script is a special list of (preprocessed) source lines
  class IFS_Script : public std::list<SourceLine>
  {
    public:
      /// Load script from file
      void load(const std::string &fileName, IFS::IFS_Control* controller);

      /// Output script content
      void display() const;

      /// Get the root file name
      const std::string& name() const;

    private:
      /// for recursive calls (\#inc)
      void load(const std::list<std::string>& includeHistory);

      /// file name of the originally loaded file
      std::string m_RootFileName;

      /// the controller which holds this script
      IFS::IFS_Control* m_pController;
  };

}

#endif

