/*
 *  (C) Robert Bosch GmbH 2008. All rights reserved, also regarding any
 *  disposal, exploitation, reproduction, editing, distribution, as
 *  well as in the event of application for industrial property rights.
 *
 *  Contact: AE/NE1-IFS <vAENE1.IFS@de.bosch.com>
 *
 * Disclaimer:
 * This Software is licensed under the following conditions. You may 
 * not use this file except in compliance with the following conditions:
 *
 * Robert Bosch GmbH hereby grants to licensee the non-exclusive and 
 * non-transferable right to use the software and accompanying 
 * documentation for the sole purpose of testing designs delivered by 
 * Robert Bosch GmbH to licensee.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY WARRANTY,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND 
 * NONINFRINGEMENT. IN NO EVENT SHALL ROBERT BOSCH GMBH BE LIABLE FOR
 * ANY DIRECT OR INDIRECT DAMAGES, INJURY OR OTHER LIABILITY, ARISING
 * OUT OR IN CONNECTION WITH THE SOFTWARE. ROBERT BOSCH GMBH SHALL
 * NOT BE LIABLE FOR THE ACCURACY OF THE DESIGNS TESTED WITH THE AID
 * OF THE SOFTWARE.
*/

#ifndef IFS_WRAPPER_MODULE_H
#define IFS_WRAPPER_MODULE_H

#include "IFS_ModuleBase.h"

#include <string>

namespace IFS
{

  /// The number of characters of the command vector
  /** Effectively this determines maximum command length
    */
  #define IFS_COSIM_COMMAND_CHANNEL_NUM_CHARS 100

  /// The number of possible interrupts to be transferred from the VHDL side
  #define IFS_COSIM_NUM_INTERRUPTS 10

  /// A wrapper for VHDL(=legacy) IFS modules
  class IFS_WrapperModule: public IFS::IFS_ModuleBase
  {
    public:
      SC_HAS_PROCESS(IFS_WrapperModule);

      /// Constructor
      IFS_WrapperModule( sc_module_name moduleName );

      /// \brief This method is used to transfer the specified string to
      ///        the attached legacy-module.
      virtual void transferLegacyCommand( const std::string& commandString );

      /// \brief This method is used to transfer data about the wait-command
      ///        to the legacy-module.
      virtual void waitCycles( unsigned int duration );

      /// \brief
      /// This method allows to define the way modules wait on external command requests.
      /// Usually used for LegacyModules only.
      virtual void waitExternalCommandRequest();

    protected:
      /// \name Signals used to communicate commands over the cosimulation-boundary
      /// @{

      /// The VHDL side signals a request for a new command
      sc_signal< bool >         request_Signal;

      /// The SystemC side signals the availability of a new command
      sc_signal< bool >         grant_Signal;

      /// Transfers the command string encoded as bitvector to the VHDL side
      sc_signal< sc_bv<IFS_COSIM_COMMAND_CHANNEL_NUM_CHARS*8> > command_Signal;

      /// The VHDL side can signal interrupts via this signal
      sc_signal< sc_bv<IFS_COSIM_NUM_INTERRUPTS> >              interrupts_Signal;

      /// \brief Signals the number of cycles to wait for (and implies the
      ///        execution of a WAIT cycles command on the VHDL side)
      sc_signal< unsigned int > waitDuration_Signal;

      /// @}

    protected:
      /// \brief
      /// This event is used to signal a request coming from the
      /// associated legacy-module. Needed by IFS_WrapperModule.
      sc_event m_ExternalCommandRequest;

      /// An SC_METHOD sensitive to request_Signal
      virtual void watchRequest();
      /// An SC_METHOD sensitive to interrupts_Signal
      virtual void watchInterrupts();
  };

}

#endif

